/** \file library_split.C
    \brief file containing a mock simulator main for testing DAKOTA in
    library mode on a split communicator */ 

#include <boost/lexical_cast.hpp>
#include "mpi.h"
#include "ParallelLibrary.H"
#include "ProblemDescDB.H" 
#include "DakotaStrategy.H" 
#include "DakotaModel.H"
#include "DakotaInterface.H"
#include "PluginParallelDirectApplicInterface.H"


/// Split MPI_COMM_WORLD, returning the comm and color
void manage_mpi(MPI_Comm& my_comm, int& color);

/// Return the appropriate DAKOTA input based on color (1 or 2)
void gen_dakota_input(const int& color, std::string& input);

/// Launch DAKOTA on passed communicator, tagging output/error with color
void run_dakota(const MPI_Comm& comm, const std::string& input, 
		const int& color);

/// Wait for and collect results from DAKOTA runs
void collect_results();

/// Set input to NIDR via string argument instead of input file
extern "C" void nidr_set_input_string(const char *);


/// Driver routine for testing library mode with partitioned
/// MPI_Comm. This test fixture requires MPI and can be run on 3--8
/// processors
int main(int argc, char* argv[])
{
  
  MPI_Init(&argc, &argv);

  // manage MPI split
  MPI_Comm my_comm;
  int color;
  manage_mpi(my_comm, color);

  // cleanup output files; avoid race condition
  std::remove("dakota.o.1");
  std::remove("dakota.o.2");
  // remove("dakota.o");
  std::remove("dakota.e.1");
  std::remove("dakota.e.2");
  //remove("dakota.e");
  sleep(1);

  // colors 1 and 2 run DAKOTA
  if (color != 0) {
    std::string input;
    gen_dakota_input(color, input);
    run_dakota(my_comm, input, color);
  }
  
  // ideally color 0 would do something concurrently...
  MPI_Barrier(MPI_COMM_WORLD);

  if (color == 0)
    collect_results();

  // ideally color 0 would do something concurrently...
  MPI_Barrier(MPI_COMM_WORLD);

  MPI_Comm_free(&my_comm);
  MPI_Finalize();

  return 0;
}


void manage_mpi(MPI_Comm& my_comm, int& color) 
{
  int world_rank, world_size;

  MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &world_size);

  if (world_size < 3) {
    std::cerr << "At least 3 processors required; " << world_size 
	      << " specified." << std::endl;
    MPI_Abort(MPI_COMM_WORLD, 1);
  } else if (world_size > 8) {
    std::cerr << "At most 8 processors allowed; " << world_size 
	      << " specified." << std::endl;
    MPI_Abort(MPI_COMM_WORLD, 1);
  }

  std::cout << "Rank " << world_rank << " of " << world_size << " starting."
	    << std::endl;

  // calculate size of DAKOTA job 1 and 2
  int s1 = static_cast<int>(std::ceil((world_size-1)/2.0));
  int s2 = static_cast<int>(std::floor((world_size-1)/2.0));

  if (world_rank == 0) {
    std::cout << "Number of processes: \n  collector: 1\n";
    std::cout << "  DAKOTA 1: " << s1 << "\n";
    std::cout << "  DAKOTA 2: " << s2 << std::endl;
  }

  if (world_rank == 0)
    color = 0;
  else if (world_rank <= s1)
    color = 1;
  else if (world_rank <= (s1 + s2))
    color = 2;
  else {
    std::cerr << "Error in color calculation; exiting." << std::endl;
    MPI_Abort(MPI_COMM_WORLD, 1);
  }

  MPI_Comm_split(MPI_COMM_WORLD, color, world_rank, &my_comm);

  int newrank;
  MPI_Comm_rank(my_comm, &newrank);
  
  std::cout << "Was rank " << world_rank << " in MPI_Comm = " << MPI_COMM_WORLD
	    << "; now rank " << newrank << " in MPI_Comm = " << my_comm 
	    << std::endl;
}


void gen_dakota_input(const int& color, std::string& input)
{

  // TODO: diagnose problems with ded. master/self scheduling which hangs
  //       mandating use of evaluation_static_scheduling
  switch(color) {

  case 1:
    // CONMIN MFD on 3 variables
    input = 
      "	method,"
      "		conmin_frcg"
      "		  max_iterations = 50"
      "		  convergence_tolerance = 1e-4"
      "	variables,"
      "		continuous_design = 3"
      "		  descriptors 'x1' 'x2' 'x3'"
      "	interface,"
      "		direct"
      "		  analysis_driver = 'plugin_text_book'"
      "           evaluation_static_scheduling"
      "	responses,"
      "		num_objective_functions = 1"
      "		numerical_gradients"
      "		no_hessians";
    break;

  case 2:
    // OPT++ QN on 2 variables
    input = 
      "	method,"
      "		optpp_q_newton"
      "		  max_iterations = 50"
      "		  convergence_tolerance = 1e-4"
      "	variables,"
      "		continuous_design = 2"
      "		  descriptors 'x1' 'x2'"
      "	interface,"
      "		direct"
      "		  analysis_driver = 'plugin_text_book'"
      "           evaluation_static_scheduling"
      "	responses,"
      "		num_objective_functions = 1"
      "		num_nonlinear_inequality_constraints = 2"
      "		numerical_gradients"
      "		no_hessians";
    break;

  default:
    std::cerr << "Unknown color " << color << "; exiting" << std::endl;
    MPI_Abort(MPI_COMM_WORLD, 1);
    break;

  }
}


void run_dakota(const MPI_Comm& my_comm, const std::string& input, 
		const int& color)
{
  std::cout << "*** Starting DAKOTA run " << color << std::endl;
  
  // construct DAKOTA instances on the communicator
  Dakota::ParallelLibrary parallel_lib(my_comm);
  Dakota::ProblemDescDB problem_db(parallel_lib);

  // override output, error, and write restart files, but not read restart
  std::string ofile("dakota.o." + boost::lexical_cast<std::string>(color));
  std::string efile("dakota.e." + boost::lexical_cast<std::string>(color));
  std::string wfile("dakota.rst." + boost::lexical_cast<std::string>(color));
  const char* const rfile = NULL;
  parallel_lib.specify_outputs_restart(ofile.c_str(), efile.c_str(),
  				       rfile, wfile.c_str());

  nidr_set_input_string(input.c_str());

  // use manage_inputs since no late updates to data
  const char* const input_filename = NULL;
  problem_db.manage_inputs(input_filename);

  // Create strategy from the problem_db
  Dakota::Strategy selected_strategy(problem_db);

  // Perform interface plug-ins.
  // retrieve the currently active analysisComm from the Model.  In the most
  // general case, need an array of Comms to cover all Model configurations.
  Dakota::ModelList& models = problem_db.model_list();
  Dakota::ModelLIter ml_iter = models.begin();
  Dakota::ModelLIter ml_end = models.end();
  for ( ; ml_iter != ml_end; ++ml_iter) {
    Dakota::Interface& interface = ml_iter->interface();
    const Dakota::ParallelLevel& ea_level
      = ml_iter->parallel_configuration_iterator()->ea_parallel_level();
    const MPI_Comm& analysis_comm = ea_level.server_intra_communicator();
    interface.assign_rep(new SIM::ParallelDirectApplicInterface(problem_db, 
								analysis_comm), 
			 false);
  }

  // Execute the strategy
  problem_db.lock(); // prevent run-time DB queries
  selected_strategy.run_strategy();

  std::cout << "*** Finished DAKOTA run " << color << std::endl;
}


void collect_results()
{
  sleep(1);  // avoid file race condition
  // for dakota_test.perl benefit; no easy way to sequence output and error
  std::system("cat dakota.o.1");
  std::system("cat dakota.o.2");
  //system("cat dakota.o.1 > dakota.o");
  //system("cat dakota.o.2 >> dakota.o");
  //system("cat dakota.e.1 > dakota.e");
  //system("cat dakota.e.2 >> dakota.e");
}
